# coding: utf-8

"""
    Debian Code Search

    OpenAPI for https://codesearch.debian.net/  # noqa: E501

    OpenAPI spec version: 1.4.0
    Contact: stapelberg@debian.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from debian_codesearch_client.swagger_client.configuration import Configuration


class SearchResult(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "package": "str",
        "path": "str",
        "line": "int",
        "context_before": "list[str]",
        "context": "str",
        "context_after": "list[str]",
    }

    attribute_map = {
        "package": "package",
        "path": "path",
        "line": "line",
        "context_before": "context_before",
        "context": "context",
        "context_after": "context_after",
    }

    def __init__(
        self,
        package=None,
        path=None,
        line=None,
        context_before=None,
        context=None,
        context_after=None,
        _configuration=None,
    ):  # noqa: E501
        """SearchResult - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._package = None
        self._path = None
        self._line = None
        self._context_before = None
        self._context = None
        self._context_after = None
        self.discriminator = None

        self.package = package
        self.path = path
        self.line = line
        if context_before is not None:
            self.context_before = context_before
        self.context = context
        if context_after is not None:
            self.context_after = context_after

    @property
    def package(self):
        """Gets the package of this SearchResult.  # noqa: E501

        The Debian source package containing this search result, including the full Debian version number.  # noqa: E501

        :return: The package of this SearchResult.  # noqa: E501
        :rtype: str
        """
        return self._package

    @package.setter
    def package(self, package):
        """Sets the package of this SearchResult.

        The Debian source package containing this search result, including the full Debian version number.  # noqa: E501

        :param package: The package of this SearchResult.  # noqa: E501
        :type: str
        """
        if self._configuration.client_side_validation and package is None:
            raise ValueError(
                "Invalid value for `package`, must not be `None`"
            )  # noqa: E501

        self._package = package

    @property
    def path(self):
        """Gets the path of this SearchResult.  # noqa: E501

        Path to the file containing the this search result, starting with `package`.  # noqa: E501

        :return: The path of this SearchResult.  # noqa: E501
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path):
        """Sets the path of this SearchResult.

        Path to the file containing the this search result, starting with `package`.  # noqa: E501

        :param path: The path of this SearchResult.  # noqa: E501
        :type: str
        """
        if self._configuration.client_side_validation and path is None:
            raise ValueError(
                "Invalid value for `path`, must not be `None`"
            )  # noqa: E501

        self._path = path

    @property
    def line(self):
        """Gets the line of this SearchResult.  # noqa: E501

        Line number containing the search result.  # noqa: E501

        :return: The line of this SearchResult.  # noqa: E501
        :rtype: int
        """
        return self._line

    @line.setter
    def line(self, line):
        """Sets the line of this SearchResult.

        Line number containing the search result.  # noqa: E501

        :param line: The line of this SearchResult.  # noqa: E501
        :type: int
        """
        if self._configuration.client_side_validation and line is None:
            raise ValueError(
                "Invalid value for `line`, must not be `None`"
            )  # noqa: E501

        self._line = line

    @property
    def context_before(self):
        """Gets the context_before of this SearchResult.  # noqa: E501

        Up to 2 full lines before the search result (see `context`).  # noqa: E501

        :return: The context_before of this SearchResult.  # noqa: E501
        :rtype: list[str]
        """
        return self._context_before

    @context_before.setter
    def context_before(self, context_before):
        """Sets the context_before of this SearchResult.

        Up to 2 full lines before the search result (see `context`).  # noqa: E501

        :param context_before: The context_before of this SearchResult.  # noqa: E501
        :type: list[str]
        """

        self._context_before = context_before

    @property
    def context(self):
        """Gets the context of this SearchResult.  # noqa: E501

        The full line containing the search result.  # noqa: E501

        :return: The context of this SearchResult.  # noqa: E501
        :rtype: str
        """
        return self._context

    @context.setter
    def context(self, context):
        """Sets the context of this SearchResult.

        The full line containing the search result.  # noqa: E501

        :param context: The context of this SearchResult.  # noqa: E501
        :type: str
        """
        if self._configuration.client_side_validation and context is None:
            raise ValueError(
                "Invalid value for `context`, must not be `None`"
            )  # noqa: E501

        self._context = context

    @property
    def context_after(self):
        """Gets the context_after of this SearchResult.  # noqa: E501

        Up to 2 full lines after the search result (see `context`).  # noqa: E501

        :return: The context_after of this SearchResult.  # noqa: E501
        :rtype: list[str]
        """
        return self._context_after

    @context_after.setter
    def context_after(self, context_after):
        """Sets the context_after of this SearchResult.

        Up to 2 full lines after the search result (see `context`).  # noqa: E501

        :param context_after: The context_after of this SearchResult.  # noqa: E501
        :type: list[str]
        """

        self._context_after = context_after

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (
                            (item[0], item[1].to_dict())
                            if hasattr(item[1], "to_dict")
                            else item
                        ),
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(SearchResult, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SearchResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SearchResult):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Debian Code Search

    OpenAPI for https://codesearch.debian.net/  # noqa: E501

    OpenAPI spec version: 1.4.0
    Contact: stapelberg@debian.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from debian_codesearch_client.swagger_client.api_client import ApiClient


class SearchApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def search(self, query, **kwargs):  # noqa: E501
        """Searches through source code  # noqa: E501

        Performs a search through the full Debian Code Search corpus, blocking until all results are available (might take a few seconds depending on the search query).  Search results are ordered by their ranking (best results come first).  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.search(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str query: The search query, for example `who knows...` (literal) or `who knows\\.\\.\\.` (regular expression). See https://codesearch.debian.net/faq for more details about which keywords are supported. The regular expression flavor is RE2, see https://github.com/google/re2/blob/master/doc/syntax.txt (required)
        :param str match_mode: Whether the query is to be interpreted as a literal (`literal`) instead of as an RE2 regular expression (`regexp`). Literal searches are faster and do not require escaping special characters, regular expression searches are more powerful.
        :return: list[SearchResult]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.search_with_http_info(query, **kwargs)  # noqa: E501
        else:
            (data) = self.search_with_http_info(query, **kwargs)  # noqa: E501
            return data

    def search_with_http_info(self, query, **kwargs):  # noqa: E501
        """Searches through source code  # noqa: E501

        Performs a search through the full Debian Code Search corpus, blocking until all results are available (might take a few seconds depending on the search query).  Search results are ordered by their ranking (best results come first).  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.search_with_http_info(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str query: The search query, for example `who knows...` (literal) or `who knows\\.\\.\\.` (regular expression). See https://codesearch.debian.net/faq for more details about which keywords are supported. The regular expression flavor is RE2, see https://github.com/google/re2/blob/master/doc/syntax.txt (required)
        :param str match_mode: Whether the query is to be interpreted as a literal (`literal`) instead of as an RE2 regular expression (`regexp`). Literal searches are faster and do not require escaping special characters, regular expression searches are more powerful.
        :return: list[SearchResult]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["query", "match_mode"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in six.iteritems(params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'" " to method search" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'query' is set
        if self.api_client.client_side_validation and (
            "query" not in params or params["query"] is None
        ):  # noqa: E501
            raise ValueError(
                "Missing the required parameter `query` when calling `search`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if "query" in params:
            query_params.append(("query", params["query"]))  # noqa: E501
        if "match_mode" in params:
            query_params.append(("match_mode", params["match_mode"]))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["api_key"]  # noqa: E501

        return self.api_client.call_api(
            "/search",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="list[SearchResult]",  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def searchperpackage(self, query, **kwargs):  # noqa: E501
        """Like /search, but aggregates per package  # noqa: E501

        The search results are currently sorted arbitrarily, but we intend to sort them by ranking eventually: https://github.com/Debian/dcs/blob/51338e934eb7ee18d00c5c18531c0790a83cb698/cmd/dcs-web/querymanager.go#L719  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.searchperpackage(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str query: The search query, for example `who knows...` (literal) or `who knows\\.\\.\\.` (regular expression). See https://codesearch.debian.net/faq for more details about which keywords are supported. The regular expression flavor is RE2, see https://github.com/google/re2/blob/master/doc/syntax.txt (required)
        :param str match_mode: Whether the query is to be interpreted as a literal (`literal`) instead of as an RE2 regular expression (`regexp`). Literal searches are faster and do not require escaping special characters, regular expression searches are more powerful.
        :return: list[PackageSearchResult]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.searchperpackage_with_http_info(query, **kwargs)  # noqa: E501
        else:
            (data) = self.searchperpackage_with_http_info(query, **kwargs)  # noqa: E501
            return data

    def searchperpackage_with_http_info(self, query, **kwargs):  # noqa: E501
        """Like /search, but aggregates per package  # noqa: E501

        The search results are currently sorted arbitrarily, but we intend to sort them by ranking eventually: https://github.com/Debian/dcs/blob/51338e934eb7ee18d00c5c18531c0790a83cb698/cmd/dcs-web/querymanager.go#L719  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.searchperpackage_with_http_info(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str query: The search query, for example `who knows...` (literal) or `who knows\\.\\.\\.` (regular expression). See https://codesearch.debian.net/faq for more details about which keywords are supported. The regular expression flavor is RE2, see https://github.com/google/re2/blob/master/doc/syntax.txt (required)
        :param str match_mode: Whether the query is to be interpreted as a literal (`literal`) instead of as an RE2 regular expression (`regexp`). Literal searches are faster and do not require escaping special characters, regular expression searches are more powerful.
        :return: list[PackageSearchResult]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["query", "match_mode"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in six.iteritems(params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method searchperpackage" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'query' is set
        if self.api_client.client_side_validation and (
            "query" not in params or params["query"] is None
        ):  # noqa: E501
            raise ValueError(
                "Missing the required parameter `query` when calling `searchperpackage`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if "query" in params:
            query_params.append(("query", params["query"]))  # noqa: E501
        if "match_mode" in params:
            query_params.append(("match_mode", params["match_mode"]))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["api_key"]  # noqa: E501

        return self.api_client.call_api(
            "/searchperpackage",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="list[PackageSearchResult]",  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



# coding: utf-8

# flake8: noqa

"""
    Debian Code Search

    OpenAPI for https://codesearch.debian.net/  # noqa: E501

    OpenAPI spec version: 1.4.0
    Contact: stapelberg@debian.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import apis into sdk package
from debian_codesearch_client.swagger_client.api.search_api import SearchApi

# import ApiClient
from debian_codesearch_client.swagger_client.api_client import ApiClient
from debian_codesearch_client.swagger_client.configuration import Configuration

# import models into sdk package
from debian_codesearch_client.swagger_client.models.package_search_result import (
    PackageSearchResult,
)
from debian_codesearch_client.swagger_client.models.search_result import SearchResult



#!/usr/bin/env python3

"""
CLI tool for Debian Code Search (https://codesearch.debian.net/)


"""

from argparse import ArgumentParser
from typing import List, Dict
import requests
import sys
import collections
import html

import debian_codesearch_client.swagger_client as swclient
from debian_codesearch_client.swagger_client.rest import ApiException

configuration = swclient.Configuration()
# API key specifically for https://salsa.debian.org/debian/codesearch-cli,
# please see https://codesearch.debian.net/apikeys/ for details.
configuration.api_key["x-dcs-apikey"] = (
    "MTYxMjczNjE4NnxkTGp0SHdDUnRuUU5WeDh2dGE0dWxtZC1MSFRwQ1o2eUJEdWs1S1VvTjlHYXBYazItUWo3U2daX2pNZ0hHWDVsajJ2TUdhUFNlZ2doU0dIMnFZQk9XNl9zLS1MOU83dTBhNGdnNzV6ZnNDWWVVYXkxVDh0WEJuLTdnUGc9fOH1MJNBLYYnkLVOmnFXFEAkXoHu1SknNqADZuJ2u8nY"
)

dcs = swclient.SearchApi(swclient.ApiClient(configuration))

PATHCOLOR = 33
DUPE_PATHCOLOR = 36

dedupe_results: List[swclient.models.search_result.SearchResult] = []


def say(quiet, msg):
    """Print messages to stderr"""
    if not quiet:
        sys.stderr.write("%s\n" % msg)


def is_excluded(chunk, exclusions) -> bool:
    for exclude in exclusions:
        if exclude in chunk.path:
            return True

    return False


def get_result_body(chunk, print_linenum, nocolor) -> str:
    body = ""
    for line in chunk.context_before or ():
        line = html.unescape(line)
        if print_linenum:
            body += "        %s\n" % line
        else:
            body += "%s\n" % line

    line = html.unescape(chunk.context)
    if print_linenum:
        body += "%7d %s\n" % (chunk.line, line)
    else:
        body += "%s\n" % line
        for line in chunk.context_after or ():
            line = html.unescape(line)
            if print_linenum:
                body += "        %s\n" % line
            else:
                body += "%s\n" % line

    return body[:-1]  # trim trailing line


def print_results(chunk, print_linenum, print_only_filenames, nocolor) -> None:
    """Print search results"""
    pathline = "path: %s" % chunk.path
    if nocolor:
        print(pathline)

    else:
        print("\033[%dm%s\033[0m" % (PATHCOLOR, pathline))

    if print_only_filenames:
        return

    print(get_result_body(chunk, print_linenum, nocolor))


def print_dedupe(print_linenum: int, print_only_filenames, nocolor) -> None:
    """amalgamate duplicate results and print summary"""

    bodies = collections.defaultdict(list)
    for chunk in dedupe_results:
        body = get_result_body(chunk, print_linenum, nocolor)
        bodies[body].append(chunk)

    for body, chunks in bodies.items():
        print_results(chunks[0], print_linenum, print_only_filenames, nocolor)

        first_path = chunks[0].path
        for chunk in chunks[1:]:
            pathline = "also: %s" % chunk.path
            if nocolor:
                print(pathline)

            else:
                for common_suffix in range(1, len(pathline)):
                    if pathline[-common_suffix:] != first_path[-common_suffix:]:
                        break

                i = 1 - common_suffix
                print(
                    "\033[%dm%s\033[%dm%s\033[0m"
                    % (PATHCOLOR, pathline[:i], DUPE_PATHCOLOR, pathline[i:])
                )
        else:
            print("")


def parse_args():
    ap = ArgumentParser()
    ap.add_argument("searchstring")
    ap.add_argument("--max-results", type=int, default=200)
    ap.add_argument("-q", "--quiet", action="store_true")
    ap.add_argument("-l", "--linenumber", action="store_true")
    ap.add_argument("--nocolor", action="store_true", help="Do not colorize output")
    ap.add_argument(
        "-n",
        "--print-filenames",
        action="store_true",
        help="Print only matching filenames, no contents",
    )
    ap.add_argument(
        "-d",
        "--dedupe",
        action="store_true",
        help="amalgamate results for the same file in different packages",
    )
    ap.add_argument(
        "-x",
        "--exclude",
        action="append",
        help="list of path fragments to exclude from results",
    )
    ap.add_argument(
        "-m",
        "--mode",
        choices=["literal", "regexp"],
        default="regexp",
        help="search mode (literal or regexp)",
    )
    ap.add_argument(
        "--per-package",
        action="store_true",
        default=False,
        help="group results per source package",
    )
    args = ap.parse_args()
    if not sys.stdout.isatty():
        args.nocolor = True

    return args


def main() -> None:
    args = parse_args()

    if args.quiet:
        requests.packages.urllib3.disable_warnings()  # type: ignore

    printed_chunks = set()
    try:
        # Searches through source code, see
        # https://codesearch.debian.net/apikeys/#/search/search
        if args.per_package:
            api_response, status, headers = dcs.searchperpackage_with_http_info(
                args.searchstring, match_mode=args.mode, _return_http_data_only=False
            )
        else:
            api_response, status, headers = dcs.search_with_http_info(
                args.searchstring, match_mode=args.mode, _return_http_data_only=False
            )
        printed = 0
        for chunk in api_response:
            if is_excluded(chunk, args.exclude or []):
                continue
            if printed == args.max_results:
                break
            printed += 1
            if args.per_package:
                chunk = chunk.results[0]
            if args.dedupe:
                dedupe_results.append(chunk)
            else:
                print_results(
                    chunk, args.linenumber, args.print_filenames, args.nocolor
                )
            printed_chunks.add((chunk.path, chunk.line))

        if args.dedupe:
            print_dedupe(args.linenumber, args.print_filenames, args.nocolor)

        say(
            args.quiet,
            "--\n%d files found." % int(headers["X-Codesearch-Filestotal"]),
        )

    except ApiException as e:
        print("Exception when calling SearchApi->search: %s\n" % e)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print("")
        sys.exit()



# coding: utf-8

"""
    Debian Code Search

    OpenAPI for https://codesearch.debian.net/  # noqa: E501

    OpenAPI spec version: 1.4.0
    Contact: stapelberg@debian.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from debian_codesearch_client.swagger_client.configuration import Configuration


class PackageSearchResult(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {"package": "str", "results": "list[SearchResult]"}

    attribute_map = {"package": "package", "results": "results"}

    def __init__(self, package=None, results=None, _configuration=None):  # noqa: E501
        """PackageSearchResult - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._package = None
        self._results = None
        self.discriminator = None

        self.package = package
        self.results = results

    @property
    def package(self):
        """Gets the package of this PackageSearchResult.  # noqa: E501

        The Debian source package for which up to 2 search results have been aggregated in `results`.  # noqa: E501

        :return: The package of this PackageSearchResult.  # noqa: E501
        :rtype: str
        """
        return self._package

    @package.setter
    def package(self, package):
        """Sets the package of this PackageSearchResult.

        The Debian source package for which up to 2 search results have been aggregated in `results`.  # noqa: E501

        :param package: The package of this PackageSearchResult.  # noqa: E501
        :type: str
        """
        if self._configuration.client_side_validation and package is None:
            raise ValueError(
                "Invalid value for `package`, must not be `None`"
            )  # noqa: E501

        self._package = package

    @property
    def results(self):
        """Gets the results of this PackageSearchResult.  # noqa: E501


        :return: The results of this PackageSearchResult.  # noqa: E501
        :rtype: list[SearchResult]
        """
        return self._results

    @results.setter
    def results(self, results):
        """Sets the results of this PackageSearchResult.


        :param results: The results of this PackageSearchResult.  # noqa: E501
        :type: list[SearchResult]
        """
        if self._configuration.client_side_validation and results is None:
            raise ValueError(
                "Invalid value for `results`, must not be `None`"
            )  # noqa: E501

        self._results = results

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (
                            (item[0], item[1].to_dict())
                            if hasattr(item[1], "to_dict")
                            else item
                        ),
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(PackageSearchResult, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PackageSearchResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, PackageSearchResult):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Debian Code Search

    OpenAPI for https://codesearch.debian.net/  # noqa: E501

    OpenAPI spec version: 1.4.0
    Contact: stapelberg@debian.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import io
import json
import logging
import re
import ssl

import certifi

# python 2 and python 3 compatibility library
import six
from six.moves.urllib.parse import urlencode

try:
    import urllib3
except ImportError:
    raise ImportError("Swagger python client requires urllib3.")


logger = logging.getLogger(__name__)


class RESTResponse(io.IOBase):

    def __init__(self, resp):
        self.urllib3_response = resp
        self.status = resp.status
        self.reason = resp.reason
        self.data = resp.data

    def getheaders(self):
        """Returns a dictionary of the response headers."""
        return self.urllib3_response.headers

    def getheader(self, name, default=None):
        """Returns a given response header."""
        return self.urllib3_response.headers.get(name, default)


class RESTClientObject(object):

    def __init__(self, configuration, pools_size=4, maxsize=None):
        # urllib3.PoolManager will pass all kw parameters to connectionpool
        # https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/poolmanager.py#L75  # noqa: E501
        # https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/connectionpool.py#L680  # noqa: E501
        # maxsize is the number of requests to host that are allowed in parallel  # noqa: E501
        # Custom SSL certificates and client certificates: http://urllib3.readthedocs.io/en/latest/advanced-usage.html  # noqa: E501

        # cert_reqs
        if configuration.verify_ssl:
            cert_reqs = ssl.CERT_REQUIRED
        else:
            cert_reqs = ssl.CERT_NONE

        # ca_certs
        if configuration.ssl_ca_cert:
            ca_certs = configuration.ssl_ca_cert
        else:
            # if not set certificate file, use Mozilla's root certificates.
            ca_certs = certifi.where()

        addition_pool_args = {}
        if configuration.assert_hostname is not None:
            addition_pool_args["assert_hostname"] = (
                configuration.assert_hostname
            )  # noqa: E501

        if maxsize is None:
            if configuration.connection_pool_maxsize is not None:
                maxsize = configuration.connection_pool_maxsize
            else:
                maxsize = 4

        # https pool manager
        if configuration.proxy:
            self.pool_manager = urllib3.ProxyManager(
                num_pools=pools_size,
                maxsize=maxsize,
                cert_reqs=cert_reqs,
                ca_certs=ca_certs,
                cert_file=configuration.cert_file,
                key_file=configuration.key_file,
                proxy_url=configuration.proxy,
                **addition_pool_args
            )
        else:
            self.pool_manager = urllib3.PoolManager(
                num_pools=pools_size,
                maxsize=maxsize,
                cert_reqs=cert_reqs,
                ca_certs=ca_certs,
                cert_file=configuration.cert_file,
                key_file=configuration.key_file,
                **addition_pool_args
            )

    def request(
        self,
        method,
        url,
        query_params=None,
        headers=None,
        body=None,
        post_params=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        """Perform requests.

        :param method: http request method
        :param url: http request url
        :param query_params: query parameters in the url
        :param headers: http request headers
        :param body: request json body, for `application/json`
        :param post_params: request post parameters,
                            `application/x-www-form-urlencoded`
                            and `multipart/form-data`
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        """
        method = method.upper()
        assert method in ["GET", "HEAD", "DELETE", "POST", "PUT", "PATCH", "OPTIONS"]

        if post_params and body:
            raise ValueError(
                "body parameter cannot be used with post_params parameter."
            )

        post_params = post_params or {}
        headers = headers or {}

        timeout = None
        if _request_timeout:
            if isinstance(
                _request_timeout, (int,) if six.PY3 else (int, long)
            ):  # noqa: E501,F821
                timeout = urllib3.Timeout(total=_request_timeout)
            elif isinstance(_request_timeout, tuple) and len(_request_timeout) == 2:
                timeout = urllib3.Timeout(
                    connect=_request_timeout[0], read=_request_timeout[1]
                )

        if "Content-Type" not in headers:
            headers["Content-Type"] = "application/json"

        try:
            # For `POST`, `PUT`, `PATCH`, `OPTIONS`, `DELETE`
            if method in ["POST", "PUT", "PATCH", "OPTIONS", "DELETE"]:
                if query_params:
                    url += "?" + urlencode(query_params)
                if re.search("json", headers["Content-Type"], re.IGNORECASE):
                    request_body = "{}"
                    if body is not None:
                        request_body = json.dumps(body)
                    r = self.pool_manager.request(
                        method,
                        url,
                        body=request_body,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers,
                    )
                elif (
                    headers["Content-Type"] == "application/x-www-form-urlencoded"
                ):  # noqa: E501
                    r = self.pool_manager.request(
                        method,
                        url,
                        fields=post_params,
                        encode_multipart=False,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers,
                    )
                elif headers["Content-Type"] == "multipart/form-data":
                    # must del headers['Content-Type'], or the correct
                    # Content-Type which generated by urllib3 will be
                    # overwritten.
                    del headers["Content-Type"]
                    r = self.pool_manager.request(
                        method,
                        url,
                        fields=post_params,
                        encode_multipart=True,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers,
                    )
                # Pass a `string` parameter directly in the body to support
                # other content types than Json when `body` argument is
                # provided in serialized form
                elif isinstance(body, str):
                    request_body = body
                    r = self.pool_manager.request(
                        method,
                        url,
                        body=request_body,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers,
                    )
                else:
                    # Cannot generate the request from given parameters
                    msg = """Cannot prepare a request message for provided
                             arguments. Please check that your arguments match
                             declared content type."""
                    raise ApiException(status=0, reason=msg)
            # For `GET`, `HEAD`
            else:
                r = self.pool_manager.request(
                    method,
                    url,
                    fields=query_params,
                    preload_content=_preload_content,
                    timeout=timeout,
                    headers=headers,
                )
        except urllib3.exceptions.SSLError as e:
            msg = "{0}\n{1}".format(type(e).__name__, str(e))
            raise ApiException(status=0, reason=msg)

        if _preload_content:
            r = RESTResponse(r)

            # In the python 3, the response.data is bytes.
            # we need to decode it to string.
            if six.PY3:
                r.data = r.data.decode("utf8")

            # log response body
            logger.debug("response body: %s", r.data)

        if not 200 <= r.status <= 299:
            raise ApiException(http_resp=r)

        return r

    def GET(
        self,
        url,
        headers=None,
        query_params=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        return self.request(
            "GET",
            url,
            headers=headers,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            query_params=query_params,
        )

    def HEAD(
        self,
        url,
        headers=None,
        query_params=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        return self.request(
            "HEAD",
            url,
            headers=headers,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            query_params=query_params,
        )

    def OPTIONS(
        self,
        url,
        headers=None,
        query_params=None,
        post_params=None,
        body=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        return self.request(
            "OPTIONS",
            url,
            headers=headers,
            query_params=query_params,
            post_params=post_params,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            body=body,
        )

    def DELETE(
        self,
        url,
        headers=None,
        query_params=None,
        body=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        return self.request(
            "DELETE",
            url,
            headers=headers,
            query_params=query_params,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            body=body,
        )

    def POST(
        self,
        url,
        headers=None,
        query_params=None,
        post_params=None,
        body=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        return self.request(
            "POST",
            url,
            headers=headers,
            query_params=query_params,
            post_params=post_params,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            body=body,
        )

    def PUT(
        self,
        url,
        headers=None,
        query_params=None,
        post_params=None,
        body=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        return self.request(
            "PUT",
            url,
            headers=headers,
            query_params=query_params,
            post_params=post_params,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            body=body,
        )

    def PATCH(
        self,
        url,
        headers=None,
        query_params=None,
        post_params=None,
        body=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        return self.request(
            "PATCH",
            url,
            headers=headers,
            query_params=query_params,
            post_params=post_params,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            body=body,
        )


class ApiException(Exception):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.data
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "({0})\n" "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message
